package org.netkernel.layer0.urii;

import java.io.File;
import java.net.URI;
import java.net.URL;

import org.netkernel.layer0.nkf.INKFExpiryFunction;

/** An expiry function for a java.io.File that will expire if the file is modifed, created or deleted
 * @author Tony Butterfield
 *
 */
public class ClassLoaderResourceExpiry implements INKFExpiryFunction
{
	private final String mPath;
	private final ClassLoader mClassLoader;
	private final File mFile;
	private final long mExpectedTimestamp;
	private final long mPollPeriod;
	
	private long mNextCheck;
	
	/**
	 * @param aFile the file to check
	 * @param aPollPeriod how often to recheck
	 */
	public ClassLoaderResourceExpiry(String aPath, ClassLoader aClassLoader, long aPollPeriod)
	{	
		if (aPath.charAt(0)=='/')
		{	aPath = aPath.substring(1);
		}
		mPath=aPath;
		mClassLoader=aClassLoader;
		
		URL url = aClassLoader.getResource(aPath);
		if (url!=null)
		{	if (url.getProtocol().equals("file"))
			{	mFile = new File(URI.create(url.toString()));
			}
			else
			{	String urlString=url.toExternalForm();
				if (urlString.startsWith("jar:file:"))
				{	int i=urlString.lastIndexOf("!/");
					String fileString=urlString.substring(4,i);
					mFile = new File(URI.create(fileString));
				}
				else
				{	mFile=null;
				}
			}
			if (mFile!=null)
			{	mExpectedTimestamp=mFile.lastModified();
			}
			else
			{	mExpectedTimestamp=-1; //expire when url stops existing
			}
		}
		else
		{	mFile=null;
			mExpectedTimestamp=0; // expire when url starts to exist
		}
		
		mPollPeriod=aPollPeriod;
		mNextCheck=1;
	}
		
	public boolean isExpired(long aNow)
	{	boolean result;
		if (mNextCheck==0)
		{	result = true;
		}
		else if (mNextCheck>aNow )
		{	result = false;
		}
		else
		{	
			if (mFile!=null)
			{	long lastModified = mFile.lastModified();
				// a zero value if the file has been deleted
				result = lastModified>mExpectedTimestamp || lastModified==0;
			}
			else
			{	URL url = mClassLoader.getResource(mPath);
				result=(url==null && mExpectedTimestamp==-1) || (url!=null && mExpectedTimestamp==0);
			}
			
			if (result)
			{	mNextCheck=0;
			}
			else
			{	mNextCheck = aNow+mPollPeriod;
			}
		}
		return result;
	}
	
	public String toString()
	{	if (mFile!=null)
		{	return mFile.getAbsolutePath();
		}
		else
		{	return "getResource(\""+mPath+"\") in "+mClassLoader.toString();
		}
	}
}
